@@ -6,9 +6,9 @@
 #' \tabular{ll}{
 #' Package: \tab rCMA\cr
 #' Type: \tab Package\cr
-#' Version: \tab 1.0.0\cr
-#' Date: \tab 01.03.2013\cr
-#' License: \tab GPL (>= 2)\cr
+#' Version: \tab 1.1\cr
+#' Date: \tab 2015-04-30\cr
+#' License: \tab GPL (>= 3)\cr
 #' LazyLoad: \tab yes\cr
 #' }
 #'
@@ -16,18 +16,20 @@
 #' CMA-ES [HansOst96, Hansen13] is the Covariance Matrix Adapting Evolutionary Strategy for numeric black box optimization. \cr 
 #' The main features of rCMA are: \enumerate{
 #'  \item Abiltiy to start the Java CMA-ES optimization with fitness functions defined in R. 
-#'  \item Constraint handling: Arbitrary constrained can be incorporated, see function parameter \code{isFeasible} 
+#'  \item Constraint handling: Arbitrary constraints can be incorporated, see function parameter \code{isFeasible} 
 #'    in \code{\link{cmaOptimDP}}. 
 #'  \item Extensibility: Full access to all methods of the Java class \code{CMAEvolutionStrategy} through package  
 #'    \href{http://cran.r-project.org/web/packages/rJava/index.html}{\code{rJava}}. New methods can be added easily.
-#'    See \code{\link{cmaEvalMeanX}} for further details, explanation of JNI types and a full example. 
-#'  \item Test and Debug: The access of Java methods from R allows for easy debugging of new R scripts. 
+#'    See the documentation of \code{\link{cmaEvalMeanX}} for further details, explanation of JNI types and a full example. 
+#'  \item Test and Debug: The access of Java methods from R allows for easy debugging and test
+#'    of programs using \code{CMAEvolutionStrategy} through R scripts without the necessity to
+#'    change the underlying JAR file.
 #' }
 #' The main entry point functions are \code{\link{cmaNew}}, \code{\link{cmaInit}} and \code{\link{cmaOptimDP}}. 
 #'
-#' Note: To install rJava properly on some Unix systmes, it might be necessary to issue as
+#' Note: To install \code{rJava} properly on some Unix systmes, it might be necessary to issue as
 #' root the command \code{R CMD javareconf} once, or, as normal user to issue the command \code{R CMD javareconf -e}
-#' prior to installing package rJava or prior to loading library rJava. 
+#' prior to installing package \code{rJava} or prior to loading library \code{rJava}. 
 #' 
 #' @name rCMA-package
 #' @aliases rCMA
@@ -39,7 +41,11 @@
 #'    In Proceedings of the 1996 IEEE International Conference on Evolutionary Computation, pp. 312-317, 1996. 
 #'    \href{https://www.lri.fr/~hansen/CMAES.pdf}{PDF}. \cr
 #'  [Hansen09] \url{https://www.lri.fr/~hansen/javadoc} Nikolaus Hansen: Javadoc for CMA-ES Java package fr.inria.optimization.cmaes, 2009. \cr
-#'  [Hansen13] \url{https://www.lri.fr/~hansen/cmaesintro.html} Nikolaus Hansen: The CMA Evolution Strategy Web Page, 2013.
+#'  [Hansen13] \url{https://www.lri.fr/~hansen/cmaesintro.html} Nikolaus Hansen: The CMA Evolution Strategy Web Page, 2013. \cr
+#'  [Urbanek13] \url{http://cran.r-project.org/web/packages/rJava}
+#'    Urbanek, S.: rJava: Low-level R to Java interface, 2013. \cr
+#'  [Oracle14] \url{http://docs.oracle.com/javase/7/docs/technotes/guides/jni/spec/jniTOC.html} 
+#'    Oracle: The Java Native Interface. Programmer's Guide and Specification, 2014.
 #' @keywords package CMA Covariance Matrix rJava
 #End of Package Description
 NA #NULL, ends description without hiding first function
@@ -51,9 +57,10 @@ NA #NULL, ends description without hiding first function
 ##    rJava::.jpackage
 ## will attach all JAR-files found in the java/-subdir of rCMA to the Java class path
 ## (see .jclassPath() )
-.onLoad <- function(libname, pkgname) {
-  rJava::.jpackage(pkgname, lib.loc=libname)
-}
+#----
+#.onLoad <- function(libname, pkgname) {
+#  rJava::.jpackage(pkgname, lib.loc=libname)
+#}
 .onAttach <- function(libname, pkgname) {
   packageStartupMessage(sprintf("Attaching %s/%s/java/cma_jAll.jar to the Java class path (.jclassPath())",libname,pkgname));
 }
@@ -79,7 +86,7 @@ NA #NULL, ends description without hiding first function
 #'    ## (see  cmaEvalMeanX-documentation for further details on .jcall and its argument "S")
 #'    cma <- cmaNew();
 #'    props <- attr(cma,"props");
-#'    initialX = .jcall(props,"S","getProperty","initialX");
+#'    initialX = rJava::.jcall(props,"S","getProperty","initialX");
 #'    print(initialX);
 #'
 #' @seealso   \code{\link{cmaInit}}
@@ -93,19 +100,19 @@ cmaNew <- function(propFile=NULL) {
   if (!file.exists(cmaJarFile)) stop(sprintf("cmaJarFile=%s not found in working directory",cmaJarFile));
   if (is.null(propFile)) propFile <- paste(find.package("rCMA"),"CMAEvolutionStrategy.properties",sep="/")
   ##
-  ## This is not necessary anymore, we have .onLoad above with .jpackage:
-  #.jinit()          # initialize Java-VM
-  #.jaddClassPath(cmaJarFile);
+  ## This is now necessary again, because the  .onLoad mechanism above does not work with R3.1.3
+  rJava::.jinit()          # initialize Java-VM
+  rJava::.jaddClassPath(cmaJarFile);
   ##
   ## For unclear reasons, the following two lines gave an error in system("R CMD check rCMA") (and only there!) 
   ## when running any example containing cmaNew(). The error was not observed when executing cmaNew() from the 
   ## R prompt directly. Anyhow, since we do not need them really, we commented the two following lines out:
   #if (length(grep("cma_jAll.jar",.jclassPath()))==0)
-  #  warning("cma_jAll.jar is not part of .jclassPath(). Check whether .jaddClassPath has access to the right directory.");
-  cma <- .jnew("fr/inria/optimization/cmaes/CMAEvolutionStrategy")
-  props <- .jcall(cma,"Ljava/util/Properties;","readProperties",propFile);
+  #  warning("cma_jAll.jar is not part of .jclassPath(). Check whether rJava::.jaddClassPath has access to the right directory.");
+  cma <- rJava::.jnew("fr/inria/optimization/cmaes/CMAEvolutionStrategy")
+  props <- rJava::.jcall(cma,"Ljava/util/Properties;","readProperties",propFile);
   ## To assess elements of props:
-  ##       initialX = .jcall(props,"S","getProperty","initialX");
+  ##       initialX = rJava::.jcall(props,"S","getProperty","initialX");
   attr(cma,"props") <- props
   cma;
 }
@@ -130,7 +137,10 @@ cmaNew <- function(propFile=NULL) {
 #'
 #' @note
 #'  As a side effect, the CMA-ES Java object \code{cma} of class \code{CMAEvolutionStrategy}
-#'  is transferred into an augmented state.
+#'  is transferred into an augmented state. As a second side effect, the population size is 
+#'  set to 
+#'        \deqn{\lambda = 4 + 3 floor(ln(n))}
+#'  where \eqn{n=}\code{dimension}.
 #'
 #' @examples
 #'    cma <- cmaNew();
@@ -141,11 +151,11 @@ cmaNew <- function(propFile=NULL) {
 #' @export
 ######################################################################################
 cmaInit <- function(cma,seed=NULL,dimension=NULL,initialX=NULL, initialStandardDeviations=NULL) {
-  if (!is.null(seed)) .jcall(cma,,"setSeed",.jlong(seed));
-  if (!is.null(dimension)) .jcall(cma,,"setDimension",as.integer(dimension));
-  if (!is.null(initialX)) .jcall(cma,,"setInitialX",.jarray(initialX));
-  if (!is.null(initialStandardDeviations)) .jcall(cma,,"setInitialStandardDeviations",.jarray(initialStandardDeviations));
-  fitness <- .jcall(cma,"[D","init");
+  if (!is.null(seed)) rJava::.jcall(cma,,"setSeed",rJava::.jlong(seed));
+  if (!is.null(dimension)) rJava::.jcall(cma,,"setDimension",as.integer(dimension));
+  if (!is.null(initialX)) rJava::.jcall(cma,,"setInitialX",rJava::.jarray(initialX));
+  if (!is.null(initialStandardDeviations)) rJava::.jcall(cma,,"setInitialStandardDeviations",rJava::.jarray(initialStandardDeviations));
+  fitness <- rJava::.jcall(cma,"[D","init");
 }
 
 ######################################################################################
@@ -161,7 +171,15 @@ cmaInit <- function(cma,seed=NULL,dimension=NULL,initialX=NULL, initialStandardD
 #' In each iteration, a population is sampled (infeasible individuals are replaced via
 #' Java function \code{resampleSingle}), its fitness vector is evaluated and the CMA 
 #' distribution is updated according to this fitness vector.
-#'
+#' 
+#' Every \code{iterPrint} generations a one-line diagnostic output of the form
+#' \preformatted{
+#'      iter  fitness | x1 x2 ... xp
+#' } 
+#' is printed where \code{fitness} is the current best value of the fitness function to be minimized
+#' and \code{x1 x2 ... xp} are the first \code{maxDimPrint} dimensions of the corresponding 
+#' best point in input space.
+#' 
 #' @param cma CMA-ES Java object, already initialized with \code{\link{cmaInit}}
 #' @param fitFunc a function to be minimized. Signature: accepts a vector \code{x}, returns a \code{double}.
 #' @param isFeasible [\code{function(x)\{TRUE\}}] a Boolean function checking the feasibility
@@ -196,7 +214,7 @@ cmaInit <- function(cma,seed=NULL,dimension=NULL,initialX=NULL, initialStandardD
 #' @example    demo/demoCMA2.R
 #'
 #' @seealso   \code{\link{cmaNew}}, \code{\link{cmaInit}}
-#' @author Wolfgang Konen, FHK, 2013
+#' @author Wolfgang Konen, FHK, 2013-2015
 #' @export
 ######################################################################################
 cmaOptimDP <- function( cma
@@ -204,23 +222,23 @@ cmaOptimDP <- function( cma
                       , isFeasible=function(x){TRUE}
                       , maxDimPrint=5, iterPrint=10, verbose=2)
 {
-  if (.jcall(cma,"D","getState")<0) stop("CMA object is not yet initialized. Use cmaInit() first.");
+  if (rJava::.jcall(cma,"D","getState")<0) stop("CMA object is not yet initialized. Use cmaInit() first.");
   cfe = 0;
   sMsg = NULL;      # holds console output as string vector
   fitnessVec=NULL;  # best-ever fitness after each iteration
   xMat = NULL;      # (nIter x dimension) matrix: row i holds bestX after iteration i
-  iter = .jcall(cma,"J","getCountIter");          # when the while-loop is never entered 
-  while(.jcall(cma,"I","getStopConditionNumber")==0) {
-      pop = .jcall(cma,"[[D","samplePopulation");
+  iter = rJava::.jcall(cma,"J","getCountIter");          # when the while-loop is never entered 
+  while(rJava::.jcall(cma,"I","getStopConditionNumber")==0) {
+      pop = rJava::.jcall(cma,"[[D","samplePopulation");
       LP = length(pop);       # population size
-      popR = sapply(pop,.jevalArray)
+      popR = sapply(pop,rJava::.jevalArray)
       ## popR is a (2 x LP)-R-matrix  with popR[,1] being the first individuum in the population
       cfe = cfe + LP;
 
       resampleFunc <- function(i) {
         while (!isFeasible(popR[,i])) {
           ## death penalty (DP): if an individual is not feasible, throw it away and sample a new one:
-          popR[,i] <<- .jcall(cma,"[D","resampleSingle",as.integer(i-1));
+          popR[,i] <<- rJava::.jcall(cma,"[D","resampleSingle",as.integer(i-1));
           ## it is important to use "<<-" here, otherwise a local copy popR[,i] is
           ## made and nothing changed in the popR of the while-loop (!)
           cfe <<- cfe+1;
@@ -228,10 +246,10 @@ cmaOptimDP <- function( cma
       }
       feasible = sapply(1:LP, resampleFunc);
       fitness = sapply(1:LP, function(i) { x=popR[,i]; fitFunc(x); });
-      iter = .jcall(cma,"J","getCountIter")    # note that "J" is the JNI field descriptor for 'long'
+      iter = rJava::.jcall(cma,"J","getCountIter")    # note that "J" is the JNI field descriptor for 'long'
       if (iter %% iterPrint == 0) {
         i = which.min(fitness);
-        dimPrint = min(maxDimPrint,.jcall(cma,"I","getDimension"));
+        dimPrint = min(maxDimPrint,rJava::.jcall(cma,"I","getDimension"));
         s=sprintf("%04d  %e | %s\n", iter, fitness[i], paste(sprintf("%11.4e",popR[1:dimPrint,i]),collapse=", "));
         sMsg = c(sMsg,s);
         if (verbose>0) {
@@ -239,21 +257,21 @@ cmaOptimDP <- function( cma
           flush.console();
         }
       }
-      .jcall(cma,,"updateDistribution",fitness);
-      fitnessVec[iter] = .jcall(cma,"D","getBestFunctionValue");
-      xMat = rbind(xMat,.jcall(cma,"[D","getBestX"));
+      rJava::.jcall(cma,,"updateDistribution",fitness);
+      fitnessVec[iter] = rJava::.jcall(cma,"D","getBestFunctionValue");
+      xMat = rbind(xMat,rJava::.jcall(cma,"[D","getBestX"));
   }
-  bestSolution = .jcall(cma,"Lfr/inria/optimization/cmaes/CMASolution;","getBestSolution");
-  ffe = .jcall(cma,"J","getCountEval")    # number of fitness function evaluations (fitFunc)
-  s1 = sprintf("Terminated due to %s\n",.jcall(cma,"[S","getStopConditionMessages"));
+  bestSolution = rJava::.jcall(cma,"Lfr/inria/optimization/cmaes/CMASolution;","getBestSolution");
+  ffe = rJava::.jcall(cma,"J","getCountEval")    # number of fitness function evaluations (fitFunc)
+  s1 = sprintf("Terminated due to %s\n",rJava::.jcall(cma,"[S","getStopConditionMessages"));
   s2 = sprintf("cfe,ffe, %%infeasible: %d %d %f\n",cfe,ffe,(cfe-ffe)/ffe);
   sMsg = c(sMsg,s1,s2);
   if (verbose>0) cat(s1,s2);
 
   res = list( sMsg=sMsg      # to print it: 'cat(sMsg)' or 'for (x in sMsg) cat(x)'
-            , bestX = .jcall(bestSolution,"[D","getX")
-            , bestFitness = .jcall(bestSolution,"D","getFitness")
-            , bestEvalNum = .jcall(bestSolution,"J","getEvaluationNumber")
+            , bestX = rJava::.jcall(bestSolution,"[D","getX")
+            , bestFitness = rJava::.jcall(bestSolution,"D","getFitness")
+            , bestEvalNum = rJava::.jcall(bestSolution,"J","getEvaluationNumber")
             , nIter = iter   # number of iterations
             , fitnessVec = fitnessVec
             , xMat = xMat
@@ -290,8 +308,8 @@ cmaOptimDP <- function( cma
 #' @export
 ######################################################################################
 cmaSamplePopulation <- function(cma) {
-      pop = .jcall(cma,"[[D","samplePopulation");
-      popR = sapply(pop,.jevalArray);
+      pop = rJava::.jcall(cma,"[[D","samplePopulation");
+      popR = sapply(pop,rJava::.jevalArray);
 }
 
 ######################################################################################
@@ -320,7 +338,7 @@ cmaSamplePopulation <- function(cma) {
 #' @export
 ######################################################################################
 cmaCalcFitness <- function(cma,popR,fitFunc) {
-      fitness = sapply(1:nrow(popR), function(i) { x=popR[,i]; fitFunc(x); });
+      fitness = sapply(1:ncol(popR), function(i) { x=popR[,i]; fitFunc(x); });
 }
 
 ######################################################################################
@@ -341,7 +359,7 @@ cmaCalcFitness <- function(cma,popR,fitFunc) {
 #' @export
 ######################################################################################
 cmaUpdateDistribution <- function(cma,fitness) {
-      .jcall(cma,,"updateDistribution",fitness);
+  rJava::.jcall(cma,,"updateDistribution",fitness);
 }
 
 
